/*
 * Quoridor server and related libraries
 * Copyright (C) 2009-2010 Jaroslav Tulach <jaroslav.tulach@apidesign.org>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. Look for COPYING file in the top folder.
 * If not, see http://www.gnu.org/licenses/.
 */

package cz.xelfi.quoridor.freemarkerdor;

import com.sun.jersey.spi.template.TemplateProcessor;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.OutputStream;
import java.io.OutputStreamWriter;
import java.io.Writer;
import java.util.Map;
import javax.script.Bindings;
import javax.script.ScriptContext;
import javax.script.ScriptEngine;
import javax.script.ScriptEngineManager;
import javax.script.ScriptException;
import javax.ws.rs.ext.Provider;
import org.openide.filesystems.FileUtil;
import org.w3c.dom.Document;

/**
 *
 * @author Jaroslav Tulach <jtulach@netbeans.org>
 */
@Provider
public class FreemarkerProcessor implements TemplateProcessor {
    private ScriptEngineManager manager;

    public String resolve(String name) {
        return name;
    }

    public void writeTo(String fqn, Object model, OutputStream out) throws IOException {
        synchronized (this) {
            if (manager == null) {
                manager = new ScriptEngineManager();
            }
        }
        ScriptEngine eng = manager.getEngineByName("freemarker");
        if (eng == null) {
            throw new IOException("freemarker not found!");
        }
        Bindings bind = eng.getContext().getBindings(ScriptContext.ENGINE_SCOPE);
        if (model instanceof Map) {
            bind.putAll((Map<? extends String, ? extends Object>)model);
        }
        if (model instanceof Document) {
            bind.put("doc", model);
        } else {
            bind.put("model", model);
        }

        Writer w = new OutputStreamWriter(out, "UTF-8");

        eng.getContext().setWriter(w);
        eng.getContext().setAttribute(ScriptEngine.FILENAME, fqn, ScriptContext.ENGINE_SCOPE);
        // silly workaround for
        // http://openide.netbeans.org/issues/show_bug.cgi?id=170177
        eng.getContext().setAttribute("org.openide.filesystems.FileObject", FileUtil.createData(FileUtil.getConfigRoot(), fqn), ScriptContext.ENGINE_SCOPE);

        InputStream is = FreemarkerProcessor.class.getResourceAsStream(fqn);
        if (is == null) {
            throw new IOException("Not found " + fqn); // NOI18N
        }
        InputStreamReader r = new InputStreamReader(is, "UTF-8");
        try {
            eng.eval(r);
        } catch (ScriptException ex) {
            System.err.println("dump data: " + model);
            ex.printStackTrace();
            throw new IOException(ex);
        }
    }
}
